@@ -1,8 +1,5 @@
 '''\
 An asynchronous shell for experimenting with pulsar on the command line.
-The shell is available for posix operative systems only. Windows support
-to come at some point.
-
 To use write a little script, lets call it ``pshell.py``::
 
     from pulsar.apps.shell import PulsarShell 
@@ -14,38 +11,39 @@ And run it::
 
     python pshell.py
     
-The shell has already ``pulsar``, ``arbiter`` and the :class:`Actor`
-class in the global dictionary::
+.. module:: pulsar
 
-    >> pulsar.__version__
-    0.2.0
-    >> arbiter.is_arbiter()
-    True
-    >> arbiter.state
+The shell has already ``pulsar``, :func:`get_actor`, :func:`spawn`,
+:func:`send` and the :class:`Actor` class in the global dictionary::
+
+    >>> pulsar.__version__
+    0.5.0
+    >>> actor = get_actor()
+    >>> actor.info_state
     'running'
-    >> arbiter.MANAGED_ACTORS
-    {}
-    >> a = arbiter.spawn(Actor)
-    >> a.is_alive()
+    >>> a = spawn()
+    >>> a.done()
     True
-    >> arbiter.MANAGED_ACTORS
-    {'3a67a186': 3a67a186}
-    >> arbiter.close_actors()
-    >> a.is_alive()
-    False
-    
+    >>> proxy = a.result
+
+
+Implementation
+~~~~~~~~~~~~~~~~~~~~
+
+.. autoclass:: pulsar.apps.shell.PulsarShell
+   :members:
+   :member-order: bysource
 '''
 import os
 import sys
 import code
-from functools import partial
 from time import time
 
 import pulsar
-from pulsar.apps import tasks
+from pulsar.utils.pep import ispy3k
 
 
-if pulsar.ispy3k:
+if ispy3k:
     def decode_line(line):
         return line
 else:   #pragma    nocover
@@ -64,16 +62,7 @@ class InteractiveConsole(code.InteractiveConsole):  #pragma    nocover
                     (sys.version, sys.platform, cprt,pulsar.__version__)
      
     def setup(self, banner=None):
-        """Closely emulate the interactive Python console.
-
-        The optional banner argument specify the banner to print
-        before the first interaction; by default it prints a banner
-        similar to the one printed by the real Python interpreter,
-        followed by the current class name in parentheses (so as not
-        to confuse this with the real interpreter -- since it's so
-        close!).
-
-        """
+        """Closely emulate the interactive Python console."""
         try:
             sys.ps1
         except AttributeError:
@@ -102,21 +91,22 @@ class InteractiveConsole(code.InteractiveConsole):  #pragma    nocover
                 self._more = self.push(line)
 
 
-class PulsarShell(tasks.CPUboundServer):
-    can_kill_arbiter = True
-    _app_name = 'pulsarshell'
-    console_class = InteractiveConsole
-    cfg_apps = ('cpubound',)
-    cfg = {'timeout':5,
-           'workers':1,
-           'loglevel':'none',
-           'concurrency':'thread'}
+class PulsarShell(pulsar.Application):
+    name = 'shell'
+    cfg = pulsar.Config(loglevel='none', process_name='Pulsar shell',
+                        console_class=InteractiveConsole)
     
-    @property
-    def console(self):
-        return self.local.console
-    
-    def handler(self):
+    def monitor_start(self, monitor):
+        monitor.cfg.set('workers', 1)
+        monitor.cfg.set('concurrency', 'thread')
+        
+    def worker_start(self, worker):  #pragma    nocover
+        '''When the worker starts, create the :attr:`Actor.thread_pool`
+with one thread only and send the :meth:`interact` method to it.'''
+        worker.create_thread_pool()
+        worker.thread_pool.apply_async(self.start_shell, (worker,))
+        
+    def start_shell(self, worker):
         imported_objects = {'pshell': self,
                             'pulsar': pulsar,
                             'get_actor': pulsar.get_actor,
@@ -131,23 +121,14 @@ class PulsarShell(tasks.CPUboundServer):
             import rlcompleter
             readline.set_completer(rlcompleter.Completer(imported_objects).complete)
             readline.parse_and_bind("tab:complete")
-        self.local.console = self.console_class(imported_objects)
-        self.console.setup()
-        return self
-
-    def monitor_start(self, monitor):
-        monitor.cfg.set('workers', 1)
-        monitor.cfg.set('concurrency', 'thread')
-        
-    def worker_start(self, worker):
-        worker.requestloop.add_callback(partial(self.worker_task, worker))
-        
-    def worker_task(self, worker):  #pragma    nocover
+        self.local.console = self.cfg.console_class(imported_objects)
+        self.local.console.setup()
+        worker.thread_pool.apply_async(self.interact, (worker,))
+                
+    def interact(self, worker):
+        '''Handled by the :attr:`Actor.thread_pool`'''
         try:
-            self.console.interact(0.5*self.cfg.timeout)
-            self.worker_start(worker)
-        except pulsar.EXIT_EXCEPTIONS:
-            worker.send('arbiter', 'stop')
-            worker.state = pulsar.ACTOR_STATES.INACTIVE
-            
-
+            self.local.console.interact(self.cfg.timeout)
+            worker.thread_pool.apply_async(self.interact, (worker,))
+        except:
+            worker.send('arbiter', 'stop')
\ No newline at end of file
