@@ -1,15 +1,80 @@
+'''
+An application implements several :class:`Job`
+classes which specify the way each :class:`pulsar.apps.tasks.task.Task` is run.
+Each :class:`Job` class is a task-factory, therefore,
+a :ref:`task <apps-taskqueue-task>` is always associated
+with one :class:`Job`, which can be of two types:
+
+* standard (:class:`Job`)
+* periodic (:class:`PeriodicJob`), a generator of scheduled tasks.
+
+.. _job-callable:
+
+Implementing jobs
+~~~~~~~~~~~~~~~~~~~~~~~
+
+To define a job is simple, subclass from :class:`Job` and implement the
+**job callable method**::
+
+    from pulsar.apps import tasks
+
+    class Addition(tasks.Job):
+
+        def __call__(self, consumer, a, b):
+            "Add two numbers"
+            return a+b
+
+The *consumer*, instance of :class:`pulsar.apps.tasks.task.TaskConsumer`,
+is passed by the :ref:`Scheduler <apps-taskqueue-scheduler>` and should
+always be the first positional argument in the callable function.
+The remaining positional arguments and/or key-valued parameters are needed by
+your job implementation.
+
+A :ref:`job callable <job-callable>` can also return a
+:ref:`coroutine <coroutine>` if it needs to perform asynchronous IO during its
+execution::
+
+    class Crawler(tasks.Job):
+
+        def __call__(self, consumer, sample, size=10):
+            response = yield http.request(...)
+            content = response.content
+            ...
+
+Job class
+~~~~~~~~~~~~~~~~~~~~~~
+
+.. autoclass:: Job
+   :members:
+   :member-order: bysource
+
+Periodic job
+~~~~~~~~~~~~~~~~~~~~~~
+
+.. autoclass:: PeriodicJob
+   :members:
+   :member-order: bysource
+
+Job registry
+~~~~~~~~~~~~~~~~~~~~~~
+
+.. autoclass:: JobRegistry
+   :members:
+   :member-order: bysource
+
+'''
 from datetime import datetime, date, timedelta
 from hashlib import sha1
 import logging
 import inspect
 
-from pulsar.utils.httpurl import iteritems
+from pulsar.utils.pep import iteritems
+from pulsar.utils.importer import import_modules
 from pulsar.utils.security import gen_unique_id
 
 
-__all__ = ['JobMetaClass','Job','PeriodicJob',
-           'anchorDate','JobRegistry','registry',
-           'create_task_id']
+__all__ = ['JobMetaClass', 'Job', 'PeriodicJob',
+           'anchorDate', 'JobRegistry', 'create_task_id']
 
 
 def create_task_id():
@@ -34,63 +99,39 @@ class JobRegistry(dict):
         instance.
 
         """
-        job = inspect.isclass(job) and job() or job
-        name = job.name
-        self[name] = job
+        if isinstance(job, JobMetaClass) and job.can_register:
+            name = job.name
+            self[name] = job()
 
     def filter_types(self, type):
         """Return a generator of all tasks of a specific type."""
         return ((job_name, job)
                     for job_name, job in iteritems(self)
                             if job.type == type)
-
-
-registry = JobRegistry()
-
+        
+    @classmethod
+    def load(cls, paths):
+        self = cls()
+        for mod in import_modules(paths):
+            for name in dir(mod):
+                self.register(getattr(mod, name))
+        return self
 
 
 class JobMetaClass(type):
-    """Metaclass for Jobs. It performs a little ammount of magic
-by:
-
-* Automatic registration of :class:`Job` instances to the
-  global :class:`JobRegistry`, unless
-  the :attr:`Job.abstract` attribute is set to ``True``.
-* If no :attr:`Job.name`` attribute is provided,
-  it is automatically set to the class name in lower case.
-* Add a logger instance with name given by 'job.name` where name is the
-  same as above.
-"""
 
     def __new__(cls, name, bases, attrs):
-        super_new = super(JobMetaClass, cls).__new__
-        job_module = attrs.get("__module__",None)
-
-        # Abstract class, remove the abstract attribute so
-        # any class inheriting from this won't be abstract by default.
-        if attrs.pop("abstract", None) or not attrs.get("autoregister", True)\
-                                       or not job_module:
-            return super_new(cls, name, bases, attrs)
-        # Automatically generate missing name.
-        job_name = attrs.get("name",name).lower()
+        attrs['can_register'] = not attrs.pop('abstract', False)
+        job_name = attrs.get("name", name).lower()
         log_prefix = attrs.get("log_prefix") or "pulsar"
-        # Because of the way import happens (recursively)
-        # we may or may not be the first time the Job tries to register
-        # with the framework. There should only be one class for each Job
-        # name, so we always return the registered version.
-        if job_name not in registry:
-            attrs["name"] = job_name
-            logname = '%s.job.%s' % (log_prefix, name)
-            attrs['logger'] = logging.getLogger(logname)
-            job_cls = super_new(cls, name, bases, attrs)
-            registry.register(job_cls)
-        return registry[job_name].__class__
+        attrs["name"] = job_name
+        logname = '%s.job.%s' % (log_prefix, name)
+        attrs['logger'] = logging.getLogger(logname)
+        return super(JobMetaClass, cls).__new__(cls, name, bases, attrs)
 
 
-JobBase = JobMetaClass('JobBase',(object,),{'abstract':True})
 
-
-class Job(JobBase):
+class Job(JobMetaClass('JobBase', (object,), {'abstract': True})):
     '''The Job class which is used in a distributed task queue.
 
 .. attribute:: name
@@ -103,11 +144,6 @@ class Job(JobBase):
     If set to ``True`` (default is ``False``), the Job won't be registered
     with the :class:`JobRegistry`.
 
-.. attribute:: autoregister
-
-    If ``False`` (default is ``True``), the Job need to be registered
-    manually with the Job registry.
-
 .. attribute:: type
 
     Type of Job, one of ``regular`` and ``periodic``.
@@ -133,32 +169,37 @@ class Job(JobBase):
 
     Default: ``markdown``
 
-.. attribute:: loglevel
-
-    Level of task logging
-
-    Default: ``None``
-
 .. attribute:: logger
 
-    an instance of a logger.
+    an instance of a logger. Created at runtime.
 '''
     abstract = True
-    autoregister = True
     type = "regular"
     timeout = None
     expires = None
-    loglevel = None
-    logformatter = None
     doc_syntax = 'markdown'
     can_overlap = True
-    _ack = True
 
     def __call__(self, consumer, *args, **kwargs):
         '''The Jobs' task executed by the consumer. This function needs to be
 implemented by subclasses.'''
         raise NotImplementedError("Jobs must define the run method.")
 
+    def run_job(self, consumer, jobname, *args, **kwargs):
+        '''Send a new task request to the task queue from within another
+:class:`Task`. This allows tasks to act as tasks factories.
+
+:parameter consumer: The :class:`TaskConsumer` handling the :class:`Task`.
+:parameter jobname: The name of the :class:`Job` to run.
+:parameter args: positional argument for the :ref:`job callable <job-callable>`.
+:parameter kwargs: key-valued parameters for the
+    :ref:`job callable <job-callable>`.
+:rtype: A :class:`pulsar.ActorMessage` if **ack=True** was passed in the
+    key-valued parameters, otherwise nothing.
+'''
+        kwargs['from_task'] = consumer.task_id
+        return consumer.backend.run_job(jobname, args, kwargs)
+
     def make_task_id(self, args, kwargs):
         '''Get the task unique identifier.
 This can be overridden by Job implementation.
@@ -184,41 +225,7 @@ Called by the :attr:`TaskQueue.scheduler` when creating a new task.
                             for k in sorted(kwargs)))
             name = '%s%s' % (self.name, suffix)
             return sha1(name.encode('utf-8')).hexdigest()[:8]
-
-    def on_same_id(self, task):
-        '''Callback invocked when a task has an id equal to a task already
-submitted to the task queue. By default return None and the
-task is aborted.'''
-        return None
-
-    def ack(self, args, kwargs):
-        '''Return ``True`` if a Job task will acknowledge the task queue
- once the result is ready or an error has occured.
- By default it returns ``True`` but it can be overridden so that its
- behaviour can change at runtime.'''
-        return self._ack
-
-    def send_to_queue(self, consumer, jobname, *args, **kwargs):
-        '''Send a new task request to the :class:`TaskQueue`
-from within another :class:`Task`.
-This allows tasks to act as tasks factories.
-
-:parameter consumer: The :class:`TaskConsumer` handling the :class:`Task`.
-:parameter jobname: The name of the :class:`Job` to run.
-:parameter args: positional argument for the :ref:`job callable <job-callable>`.
-:parameter kwargs: key-valued parameters for the
-    :ref:`job callable <job-callable>`.
-:rtype: A :class:`pulsar.ActorMessage` if **ack=True** was passed in the
-    key-valued parameters, otherwise nothing.
-'''
-        worker = consumer.worker
-        ack = kwargs.pop('ack', False)
-        oper = "addtask" if ack else "addtask_noack"
-        res = worker.send(worker.monitor, oper, jobname,
-                          {'from_task': consumer.task.id},
-                           *args, **kwargs)
-        return res if ack else None
-
+        
 
 class PeriodicJob(Job):
     '''A periodic :class:`Job` implementation.'''
@@ -251,9 +258,10 @@ You can override this to decide the interval at runtime.
 
 
 
-def anchorDate(hour = 0, minute = 0, second = 0):
+def anchorDate(hour=0, minute=0, second=0):
+    '''Create an anchor date.'''
     td = date.today()
-    return datetime(year = td.year, month = td.month, day = td.day,
-                    hour = hour, minute = minute, second = second)
+    return datetime(year=td.year, month=td.month, day=td.day,
+                    hour=hour, minute=minute, second=second)
 
 
