@@ -38,14 +38,15 @@ WSGI Middleware
 =====================
 
 Middleware are function or callable objects similar to :ref:`wsgi-handlers`
-with the only difference that they can return ``None``. Middleware is used
-in conjunction with both :class:`WsgiHandler`.
+with the only difference that they can return ``None``. Middleware can be used
+in conjunction with :class:`WsgiHandler` or any other handler which iterate
+through middleware in a similar way (for example django wsgi handler).
 Here we introduce the :class:`Router` and :class:`MediaRouter` which handle
 requests on given urls. Pulsar is shipped with
 :ref:`additional wsgi middleware <wsgi-additional-middleware>` for manipulating
 the environment before a client response is returned.
 
-.. _apps-wsgi-router:
+.. _wsgi-router:
 
 Router
 ~~~~~~~~~~~~~~~~~~
@@ -102,10 +103,11 @@ import mimetypes
 from functools import partial
 from email.utils import parsedate_tz, mktime_tz
 
-from pulsar.utils.httpurl import http_date, CacheControl, remove_double_slash
+from pulsar.utils.httpurl import http_date, CacheControl
 from pulsar.utils.structures import AttributeDictionary, OrderedDict
 from pulsar.utils.log import LocalMixin, local_property
-from pulsar import Http404, PermissionDenied, HttpException, HttpRedirect, async, is_async
+from pulsar import Http404, PermissionDenied, HttpException, HttpRedirect,\
+                    async, is_async, multi_async
 
 from .route import Route
 from .utils import wsgi_request
@@ -231,8 +233,8 @@ class Redirect(object):
     
     
 class Router(RouterType('RouterBase', (object,), {})):
-    '''A WSGI application which handle multiple
-:ref:`routes <apps-wsgi-route>`. A user must implement the HTTP method
+    '''A WSGI middleware to handle client requests on multiple
+:ref:`routes <apps-wsgi-route>`. The user must implement the HTTP methods
 required by her application. For example if the route needs to serve a ``GET``
 request, the ``get(self, request)`` method must be implemented.
 
@@ -367,9 +369,10 @@ the best match.'''
         else:
             return self, match
     
+    @async()
     def response(self, environ, start_response, args):
         '''Once the :meth:`resolve` method has matched the correct
-:class:`Router` for serving the request, this matched roter invokes this method
+:class:`Router` for serving the request, this matched router invokes this method
 to actually produce the WSGI response.'''
         request = wsgi_request(environ, self, args)
         # Set the response content type
@@ -379,7 +382,9 @@ to actually produce the WSGI response.'''
         if callable is None:
             raise HttpException(status=405,
                                 msg='Method "%s" not allowed' % method)
-        return callable(request)
+        # make sure cache does not contain asynchronous data
+        environ['pulsar.cache'] = yield multi_async(request.cache)
+        yield callable(request)
     
     def add_child(self, router):
         '''Add a new :class:`Router` to the :attr:`routes` list. If this
