@@ -208,7 +208,7 @@ class dump_environ(object):
         return '\n%s\n' % '\n'.join(('%s = %s' % (k, v) for k, v in env))
     
     
-def handle_wsgi_error(environ, trace=None):
+def handle_wsgi_error(environ, failure):
     '''The default handler for errors while serving an Http requests.
 
 :parameter environ: The WSGI environment.
@@ -218,13 +218,11 @@ def handle_wsgi_error(environ, trace=None):
 :parameter encoding: Optional charset.
 :return: a :class:`WsgiResponse`
 '''
-    if not trace:
-        trace = sys.exc_info()
-    error = trace[1]
     request = wsgi_request(environ)
     response = request.response
     content_type = response.content_type
     content = None
+    error = failure.error
     if isinstance(error, HTTPError):
         response.status_code = error.code or 500
     else:
@@ -232,12 +230,14 @@ def handle_wsgi_error(environ, trace=None):
         response.headers.update(getattr(error, 'headers', None) or ())
     path = ' @ path "%s"' % environ.get('PATH_INFO','/')
     e = dump_environ(environ)
+    exc_info = failure.exc_info
+    failure.clear()
     if response.status_code == 500:
         LOGGER.critical('Unhandled exception during WSGI response %s.%s',
-                        path, e, exc_info=trace)
+                        path, e, exc_info=exc_info)
     else:
         LOGGER.warning('WSGI %s status code %s.', response.status_code, path)
-        LOGGER.debug('%s', e, exc_info=trace)
+        LOGGER.debug('%s', e, exc_info=exc_info)
     if has_empty_content(response.status_code) or\
             response.status_code in REDIRECT_CODES:
         response.content_type = None
@@ -245,7 +245,7 @@ def handle_wsgi_error(environ, trace=None):
         request.cache.pop('html_document', None)
         renderer = environ.get('error.handler') or render_error
         try:
-            content = renderer(request, trace)
+            content = renderer(request, exc_info)
             if is_failure(content):
                 content.log()
                 content = None
