@@ -3,7 +3,7 @@ import logging
 import traceback
 from collections import deque, namedtuple, Mapping
 from itertools import chain
-from inspect import isgenerator, isfunction, ismethod, istraceback
+from inspect import isgenerator, istraceback, isclass
 
 from pulsar.utils import events
 from pulsar.utils.pep import raise_error_trace, iteritems, default_timer,\
@@ -30,7 +30,8 @@ __all__ = ['Deferred',
            'maybe_async',
            'async',
            'multi_async',
-           'async_sleep']
+           'async_sleep',
+           'safe_async']
 
 if ispy3k:
     from concurrent.futures._base import Error, CancelledError, TimeoutError
@@ -74,12 +75,19 @@ def is_generalised_generator(value):
 inspect.isgenerator function.'''
     return hasattr(value, '__iter__') and not hasattr(value, '__len__')
 
-def is_stack_trace(trace):
-    if isinstance(trace, remote_stacktrace):
+def is_exc_info_error(exc_info):
+    if isinstance(exc_info, remote_stacktrace):
         return True
-    elif isinstance(trace, tuple) and len(trace) == 3:
-        return istraceback(trace[2]) or\
-                 (trace[2] is None and isinstance(trace[1], trace[0]))
+    elif isinstance(exc_info, tuple) and len(exc_info) == 3:
+        trace = exc_info[2]
+        if not istraceback(trace):
+            err_class = exc_info[0]
+            if trace is None and isclass(err_class) and\
+                    issubclass(err_class, BaseException):
+                return isinstance(exc_info[1], err_class)
+            return False
+        else:
+            return True
     return False
     
 def multi_async(iterable, **kwargs):
@@ -99,9 +107,14 @@ def is_async(obj):
     '''Return ``True`` if *obj* is an asynchronous object'''
     return isinstance(obj, Deferred)
 
-def is_failure(obj):
-    '''Return ``True`` if *obj* is a failure'''
-    return isinstance(obj, Failure)
+def is_failure(obj, classes=None):
+    '''Check if ``obj`` is a :class:`Failure`. If optional ``classes``
+are given, it checks if the error is an instance of those classes.'''
+    if isinstance(obj, Failure):
+        if classes:
+            return obj.isinstance(classes)
+        return True
+    return False
 
 def default_maybe_failure(value, msg=None):
     if isinstance(value, BaseException):
@@ -110,7 +123,7 @@ def default_maybe_failure(value, msg=None):
             return Failure(exc_info, msg)
         else:
             return Failure((value.__class__, value, None), msg)
-    elif is_stack_trace(value):
+    elif is_exc_info_error(value):
         return Failure(value, msg)
     else:
         return value
@@ -176,7 +189,6 @@ with pulsar :class:`Deferred` and :class:`Failure`.'''
     _maybe_async = maybe_async_callable
     _maybe_failure = maybe_failure_callable
     
-
 def async_sleep(timeout):
     '''The asynchronous equivalent of ``time.sleep(timeout)``. Use this
 function within a :ref:`coroutine <coroutine>` when you need to resume
@@ -204,36 +216,55 @@ Typical usage::
     @async()
     def myfunction(...):
         ...
+        
+It can also be used to safely call functions::
+
+    async()(myfunction, ...)
+    
+This syntax will always return a :class:`Deferred`::
+
+    async(get_result=False)(myfunction, ...)
 '''
     def __init__(self, **params):
          self.params = params
 
     def __call__(self, func):
         def _(*args, **kwargs):
-            try:
-                result = func(*args, **kwargs)
-            except Exception:
-                result = sys.exc_info()
-            return maybe_async(result, **self.params)
+            return self.call(func, *args, **kwargs)
         _.__name__ = func.__name__
         _.__doc__ = func.__doc__
         _.async = True
         return _
+    
+    def call(self, callable, *args, **kwargs):
+        '''Safely execute a ``callable`` and always return a :class:`Deferred`,
+even if the ``callable`` is not asynchronous. Never throws.'''
+        try:
+            result = callable(*args, **kwargs)
+        except Exception:
+            result = sys.exc_info()
+        return maybe_async(result, **self.params)
+
+
+safe_async = async(get_result=False).call
 
         
 ############################################################### FAILURE
 class Failure(object):
-    '''Aggregate errors during :class:`Deferred` callbacks.
+    '''The asynchronous equivalent of python Exception. It accumulate errors
+during :class:`Deferred` callbacks.
 
 .. attribute:: traces
 
     List of (``errorType``, ``errvalue``, ``traceback``) occured during
-    the execution of a :class:`Deferred`.
+    the execution of a :class:`Deferred`. Usually, this list contains
+    one element only.
 
 .. attribute:: logged
 
-    Check if the last error was logged. It can be a way ofm switching off
+    Check if the :attr:`error` was logged. It can be a way of switching off
     logging for certain errors.
+    
 '''
     def __init__(self, err=None, msg=None):
         self.msg = msg or ''
@@ -243,31 +274,52 @@ class Failure(object):
     def __repr__(self):
         return '\n\n'.join(self.format_all())
     __str__ = __repr__
-
     
+    #def __del__(self):
+    #    self.log('Deferred Failure never retrieved')
+
     def _get_logged(self):
-        return getattr(self.trace[1], '_failure_logged', False)
+        return getattr(self.error, '_failure_logged', False)
     def _set_logged(self, value):
-        err = self.trace[1]
+        err = self.error
         if err:
             setattr(err, '_failure_logged', value)
     logged = property(_get_logged, _set_logged)
     
-    def append(self, trace):
-        '''Add new failure to self.'''
-        if trace:
-            if is_failure(trace):
-                self.traces.extend(trace.get_traces())
-            elif isinstance(trace, Exception):
+    @property
+    def trace(self):
+        if self.traces:
+            return self.traces[-1]
+        else:
+            return (None, None, None)
+    
+    @property
+    def exc_info(self):
+        if self.traces:
+            return self.traces[-1]
+        else:
+            return (None, None, None)
+    
+    @property
+    def error(self):
+        '''Last python :class:`Exception` instance if available.'''
+        return self.trace[1]
+    
+    def append(self, failure):
+        '''Append new ``failure`` to self. The input ``failure`` can be another
+:class:`Failure`, an :class:`Exception` or a system exception info tuple
+obtained from ``sys.exc_info()``.'''
+        if failure:
+            if is_failure(failure):
+                self.traces.extend(failure.traces)
+            elif isinstance(failure, Exception):
                 self.traces.append(sys.exc_info())
-            elif is_stack_trace(trace):
-                self.traces.append(trace)
+            elif is_exc_info_error(failure):
+                self.traces.append(failure)
         return self
-
-    def get_traces(self):
-        return self.traces
     
     def clear(self):
+        '''Clear this :class:`Failure`'''
         self.traces = []
 
     def format_all(self):
@@ -279,8 +331,9 @@ class Failure(object):
             else:
                 yield str(value)
     
-    def is_instance(self, classes):
-        return isinstance(self.trace[1], classes)
+    def isinstance(self, classes):
+        '''Check if :attr:`error` is an instance of exception ``classes``.'''
+        return isinstance(self.error, classes)
             
     def __getstate__(self):
         self.log()
@@ -318,23 +371,15 @@ class Failure(object):
                 raise DeferredFailure(
                     'There were {0} failures during callbacks.'.format(N))
 
-    @property
-    def trace(self):
-        if self.traces:
-            return self.traces[-1]
-        else:
-            return (None, None, None)
-
     def log(self, log=None, msg=None, level=None):
-        if not self.logged:
-            self.logged = True
+        if self.traces and not self.logged:
             log = log or LOGGER
-            msg=  msg or self.msg
-            for e in self:
-                if level:
-                    getattr(log, level)(msg)
-                else:
-                    log.error(msg, exc_info=e)
+            msg = msg or self.msg
+            if level:
+                getattr(log, level)(msg)
+            else:
+                log.error(msg, exc_info=self.trace)
+        self.logged = True
 
 
 ############################################################### Deferred
@@ -379,11 +424,7 @@ which will be put off until later. It conforms with the
         self._description = description
         self._callbacks = deque()
         self._event_loop = event_loop
-        if timeout and timeout > 0:
-            # create the timeout. We don't cancel the timeout after
-            # a callback is received since the result may be still asynchronous
-            self._timeout = self.event_loop.call_later(timeout, self.cancel,
-                                            'timeout (%s seconds)' % timeout)
+        self.set_timeout(timeout)
 
     def __repr__(self):
         v = self._description or self.__class__.__name__
@@ -402,6 +443,24 @@ which will be put off until later. It conforms with the
 can be set during initialisation.'''
         return self._event_loop or get_event_loop()
     
+    @property
+    def timeout(self):
+        '''The :class:`TimedCall` which handles the timeout of this
+:class:`Deferred`. Available only when a timeout is set.'''
+        return self._timeout
+    
+    def set_timeout(self, timeout):
+        '''Set a the :attr:`timeout` for this :class:`Deferred`. It returns
+``self`` so that other methods can be concatenated.'''
+        if timeout and timeout > 0:
+            if self._timeout:
+                self._timeout.cancel()
+            # create the timeout. We don't cancel the timeout after
+            # a callback is received since the result may be still asynchronous
+            self._timeout = self.event_loop.call_later(timeout, self.cancel,
+                                            'timeout (%s seconds)' % timeout)
+        return self
+        
     def cancelled(self):
         '''pep-3156_ API method, it returns ``True`` if the :class:`Deferred`
 was cancelled.'''
@@ -680,7 +739,7 @@ function when a generator is passed as argument.'''
             try:
                 result = self.gen.send(result)
             except StopIteration:
-                return self._conclude(result)
+                break
             except Exception:
                 result = sys.exc_info()
             result = maybe_async(result, event_loop=self.event_loop)
@@ -688,6 +747,8 @@ function when a generator is passed as argument.'''
                 return result.add_both(self._restart)
             elif result == NOT_DONE:
                 return self.event_loop.call_soon(self._consume, None)
+        if not self.done():
+            return self._conclude(result)
 
     def _restart(self, result):
         #restart the coroutine in the same event loop it was started
