@@ -86,8 +86,7 @@ it is created by :meth:`EventLoop.call_soon`, :meth:`EventLoop.call_later`,
 .. attribute:: cancelled
 
     Flag indicating this callback is cancelled.
-    """
-
+"""
     def __init__(self, deadline, callback, args):
         self.reschedule(deadline)
         self._callback = callback
@@ -327,6 +326,8 @@ event loop is the place where most asynchronous operations are carried out.
 """
     # Never use an infinite timeout here - it can stall epoll
     poll_timeout = 0.5
+    tid = None
+    pid = None
 
     def __init__(self, io=None, logger=None, poll_timeout=None, timer=None,
                  iothreadloop=True):
