@@ -2,66 +2,50 @@
 class PulsarException(Exception):
     '''Base class of all Pulsar Exception'''
 
-class Timeout(PulsarException):
-    '''Raised when a timeout occurs'''
-    def __init__(self, msg, timeout=None):
-        self.timeout = timeout
-        if timeout:
-            msg = msg + ' Timeout {0} surpassed.'.format(self.timeout)
-        super(Timeout,self).__init__(msg)
 
 class ImproperlyConfigured(PulsarException):
     '''A :class:`PulsarException` raised when pulsar has inconsistent
 configuration.'''
     pass
 
-class PulsarInternetException(PulsarException):
-    '''base class of all Pulsar Internet Exception'''
-    pass
-
-class MailboxError(PulsarException):
-    pass
-
-class ActorAlreadyStarted(PulsarException):
-    '''A :class:`PulsarException` raised when trying to start an actor already started'''
-
-class HaltServer(PulsarInternetException):
-
-    def __init__(self, reason='Halt', signal=None):
-        super(HaltServer,self).__init__(reason)
-        self.signal = signal
-
 
 class CommandError(PulsarException):
-    '''Exception raised when executing a Command'''
     pass
 
 
-class CommandNotFound(PulsarException):
+class CommandNotFound(CommandError):
 
     def __init__(self, name):
         super(CommandNotFound, self).__init__(
                             'Command "%s" not available' % name)
+        
+class ProtocolError(Exception):
+    '''Raised when the protocol encounter unexpected data. It will close
+the socket connection.'''
 
-class AuthenticationError(PulsarException):
+
+class TooManyConnections(Exception):
+    '''Raised when there are too many concurrent connections.'''
+    
+    
+class AuthenticationError(Exception):
     pass
 
 
-class ConnectionError(PulsarInternetException):
+class ConnectionError(Exception):
     pass
 
 
-class CouldNotParse(PulsarInternetException):
-    pass
+class ActorAlreadyStarted(PulsarException):
+    '''A :class:`PulsarException` raised when trying to start an actor already started'''
 
 
-class DeferredFailure(PulsarException):
-    pass
+class HaltServer(BaseException):
 
-
-class AlreadyCalledError(PulsarException):
-    '''Raised when a :class:`Deferred` instance receives more than
-one :meth:`Deferred.callback`.'''
+    def __init__(self, reason='Halt', signal=None, exit_code=1):
+        super(HaltServer,self).__init__(reason)
+        self.exit_code = exit_code
+        self.signal = signal
 
 
 class CannotCallBackError(PulsarException):
@@ -79,19 +63,27 @@ class NotRegisteredWithServer(PulsarException):
 class HttpException(Exception):
     status = 500
     def __init__(self, msg='', status=None, handler=None, strict=False,
-                 headers=None):
+                 headers=None, content_type=None):
         self.status = status or self.status
+        self.code = self.status # for compatibility with HTTPError
         self.handler = handler
         self.strict = strict
         self.headers = headers
-        super(HttpException,self).__init__(msg)
+        self.content_type = content_type
+        super(HttpException, self).__init__(msg)
+        
+    @property
+    def hdrs(self):
+        return self.headers
 
 
 class HttpRedirect(HttpException):
     status = 302
-    def __init__(self, location, status=None):
+    def __init__(self, location, status=None, headers=None, **kw):
+        headers = [] if headers is None else headers
+        headers.append(('location', location))
         super(HttpRedirect, self).__init__(status=status or self.status,
-              headers=[('location', location)])
+                                           headers=headers, **kw)
 
 
 class PermissionDenied(HttpException):
